# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <falcon@gnu.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import pytz
from datetime import datetime
from trytond.pool import Pool
from trytond.transaction import Transaction
from trytond.report import Report

__all__ = ['SurgeryReport']


class SurgeryReport(Report):
    __name__ = 'surgery'

    @classmethod
    def parse(cls, report, objects, data, localcontext):
        Company = Pool().get('company.company')

        timezone = None
        company_id = Transaction().context.get('company')
        if company_id:
            company = Company(company_id)
            if company.timezone:
                timezone = pytz.timezone(company.timezone)

        dt = datetime.now()

        return super(SurgeryReport, cls).parse(report, objects, data, 
            localcontext)



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from trytond.model import ModelView, ModelSQL, fields, Unique
from trytond.transaction import Transaction
from trytond.pool import Pool
from datetime import datetime
from trytond.pyson import Eval, Not, Bool, PYSONEncoder, Equal, And, Or
from trytond import backend
from sql import *

__all__ = ['Newborn', 'NeonatalApgar', 'NeonatalMedication',
    'NeonatalCongenitalDiseases', 'PediatricSymptomsChecklist']


class Newborn(ModelSQL, ModelView):
    'Newborn Information'
    __name__ = 'gnuhealth.newborn'

    STATES = {'readonly': Eval('state') == 'signed'}

    name = fields.Char('Newborn ID', states = STATES)
    patient = fields.Many2One(
        'gnuhealth.patient', 'Baby', required=True, states = STATES,
        help="Patient associated to this newborn")

    mother = fields.Many2One('gnuhealth.patient', 'Mother', states = STATES)
    newborn_name = fields.Char('Name at Birth', states = STATES)
    birth_date = fields.DateTime('DoB', required=True,
        help="Date and Time of birth", states = STATES)
    photo = fields.Binary('Picture', states = STATES)

    # Sex / Gender at birth.

    sex = fields.Selection([
        ('m', 'Male'),
        ('f', 'Female'),
        ], 'Sex',sort=False, required=True,
            help="Sex at birth. It might differ from the current patient" \
            " gender. This is the biological sex.", states = STATES)

    state = fields.Selection([
        (None, ''),
        ('draft', 'draft'),
        ('signed', 'Signed'),
        ], 'State', readonly=True, sort=False)

    cephalic_perimeter = fields.Integer('CP',
        help="Cephalic Perimeter in centimeters (cm)", states = STATES)
    length = fields.Integer('Length',
        help="Length in centimeters (cm)", states = STATES)
    weight = fields.Integer('Weight',
        help="Weight in grams (g)", states = STATES)
    apgar1 = fields.Integer('APGAR 1st minute', states = STATES)
    apgar5 = fields.Integer('APGAR 5th minute', states = STATES)
    apgar_scores = fields.One2Many('gnuhealth.neonatal.apgar', 'name',
        'APGAR scores', states = STATES)
    meconium = fields.Boolean('Meconium', states = STATES)

    #Deprecated. Use Patient conditions directly
    congenital_diseases = fields.One2Many('gnuhealth.patient.disease',
        'newborn_id', 'Congenital diseases', states = STATES)

    reanimation_stimulation = fields.Boolean('Stimulation', states = STATES)
    reanimation_aspiration = fields.Boolean('Aspiration', states = STATES)
    reanimation_intubation = fields.Boolean('Intubation', states = STATES)
    reanimation_mask = fields.Boolean('Mask', states = STATES)
    reanimation_oxygen = fields.Boolean('Oxygen', states = STATES)
    test_vdrl = fields.Boolean('VDRL', states = STATES)
    test_toxo = fields.Boolean('Toxoplasmosis', states = STATES)
    test_chagas = fields.Boolean('Chagas', states = STATES)
    test_billirubin = fields.Boolean('Billirubin', states = STATES)
    test_audition = fields.Boolean('Audition', states = STATES)
    test_metabolic = fields.Boolean('Metabolic ("heel stick screening")',
        help="Test for Fenilketonuria, Congenital Hypothyroidism, "
        "Quistic Fibrosis, Galactosemia", states = STATES)
    neonatal_ortolani = fields.Boolean('Positive Ortolani', states = STATES)
    neonatal_barlow = fields.Boolean('Positive Barlow', states = STATES)
    neonatal_hernia = fields.Boolean('Hernia', states = STATES)
    neonatal_ambiguous_genitalia = fields.Boolean('Ambiguous Genitalia',
        states = STATES)
    neonatal_erbs_palsy = fields.Boolean('Erbs Palsy', states = STATES)
    neonatal_hematoma = fields.Boolean('Hematomas', states = STATES)
    neonatal_talipes_equinovarus = fields.Boolean('Talipes Equinovarus',
        states = STATES)
    neonatal_polydactyly = fields.Boolean('Polydactyly', states = STATES)
    neonatal_syndactyly = fields.Boolean('Syndactyly', states = STATES)
    neonatal_moro_reflex = fields.Boolean('Moro Reflex', states = STATES)
    neonatal_grasp_reflex = fields.Boolean('Grasp Reflex', states = STATES)
    neonatal_stepping_reflex = fields.Boolean('Stepping Reflex',
        states = STATES)
    neonatal_babinski_reflex = fields.Boolean('Babinski Reflex',
        states = STATES)
    neonatal_blink_reflex = fields.Boolean('Blink Reflex', states = STATES)
    neonatal_sucking_reflex = fields.Boolean('Sucking Reflex', states = STATES)
    neonatal_swimming_reflex = fields.Boolean('Swimming Reflex',
        states = STATES)
    neonatal_tonic_neck_reflex = fields.Boolean('Tonic Neck Reflex',
        states = STATES)
    neonatal_rooting_reflex = fields.Boolean('Rooting Reflex',
        states = STATES)
    neonatal_palmar_crease = fields.Boolean('Transversal Palmar Crease',
        states = STATES)
    
    #Deprecated. Use Patient medication direcly
    medication = fields.One2Many('gnuhealth.patient.medication',
        'newborn_id', 'Medication')
    
    healthprof = fields.Many2One('gnuhealth.healthprofessional',
        'Health Professional',
        help="Health professional", readonly=True)

    signed_by = fields.Many2One(
        'gnuhealth.healthprofessional', 'Signed by', readonly=True,
        states={
            'invisible': Not(Equal(Eval('state'), 'signed'))
            },
        help="Health Professional that signed this document")
        
    dismissed = fields.DateTime('Discharged', states = STATES)
    notes = fields.Text('Notes', states = STATES)

    # Deprecated. the following fields will be removed in 2.8
    # Decease information on fetus / newborn are linked now in 
    # the obstetrics evaluation (prenatal) or patient if result of pregnancy
    # was a live birth.
    # The information is no longer shown at the view.
    # Fields to be removed : bd, died_at_delivery, died_at_the_hospital
    # died_being_transferred, tod, cod
    
    bd = fields.Boolean('Stillbirth')
    died_at_delivery = fields.Boolean('Died at delivery room')
    died_at_the_hospital = fields.Boolean('Died at the hospital')
    died_being_transferred = fields.Boolean('Died being transferred',
        help="The baby died being transferred to another health institution")
    tod = fields.DateTime('Time of Death')
    cod = fields.Many2One('gnuhealth.pathology', 'Cause of death')


    @staticmethod
    def default_healthprof():
        pool = Pool()
        HealthProf= pool.get('gnuhealth.healthprofessional')
        healthprof = HealthProf.get_health_professional()
        return healthprof

    @staticmethod
    def default_state():
        return 'draft'


    @classmethod
    def __setup__(cls):
        super(Newborn, cls).__setup__()

        t = cls.__table__()
        cls._sql_constraints = [
            ('name_uniq', Unique(t, t.name),
             'The Newborn ID must be unique'),
            ('patient_uniq', Unique(t, t.patient),
             'There is already a newborn record for this patient'),
            ]

        cls._buttons.update({
            'sign_newborn': {'invisible': Equal(Eval('state'), 'signed')}
            })

    @classmethod
    @ModelView.button
    def sign_newborn(cls, newborns):
        pool = Pool()
        HealthProfessional = pool.get('gnuhealth.healthprofessional')
        Appointment = pool.get('gnuhealth.appointment')
        
        newborn_id = newborns[0]

        patient_app=[]
        
        # Change the state of the newborn to "Done"

        signing_hp = HealthProfessional.get_health_professional()
        
        cls.write(newborns, {
            'state': 'signed',
            'signed_by': signing_hp,
            })


    @classmethod
    def write(cls, newborns, values):
        pool = Pool()

        cursor = Transaction().connection.cursor() 
        Patient = pool.get('gnuhealth.patient')
        Party = pool.get('party.party')

        party = []
        patient = []
        
        cursor = Transaction().connection.cursor() 


        for newborn in newborns:
            
            newborn_patient_id = newborn.patient.id
            
            person = Patient.browse([newborn_patient_id])[0].name
            pat = Patient.browse([newborn_patient_id])[0]

            # Update the birth date on the party model upon WRITING it on the 
            # newborn model

            born_date = datetime.date(newborn.birth_date)

            party.append(person)
            
            Party.write(party, {
            'dob': born_date })

            # Update the biological sex on the patient model upon WRITING 
            # it on the newborn model 

            if values.get('sex'):
                biological_sex = values.get('sex')

                patient.append(pat)
            
                Patient.write(patient, {
                'biological_sex': biological_sex })

        return super(Newborn, cls).write(newborns, values)


    @classmethod
    def create(cls, vlist):
        pool = Pool()
        vlist = [x.copy() for x in vlist]
        Patient = pool.get('gnuhealth.patient')
        Party = pool.get('party.party')

        party = []
        patient = []
        
        cursor = Transaction().connection.cursor()

        for values in vlist:
            newborn_patient_id = values['patient']

            person = Patient.browse([newborn_patient_id])[0].name
            pat = Patient.browse([newborn_patient_id])[0]
            
            # Update the birth date on the party model upon CREATING it on the 
            # newborn model

            born_date = datetime.date(values['birth_date'])

            party.append(person)
            
            Party.write(party, {
            'dob': born_date })

            # Update the biological sex on the patient model upon CREATING 
            # it on the newborn model 

            if values.get('sex'):
                biological_sex = values.get('sex')

                patient.append(pat)
            
                Patient.write(patient, {
                'biological_sex': biological_sex })
        
        return super(Newborn, cls).create(vlist)


class NeonatalApgar(ModelSQL, ModelView):
    'Neonatal APGAR Score'
    __name__ = 'gnuhealth.neonatal.apgar'

    name = fields.Many2One('gnuhealth.newborn', 'Newborn ID')

    apgar_minute = fields.Integer('Minute', required=True)

    apgar_appearance = fields.Selection([
        ('0', 'central cyanosis'),
        ('1', 'acrocyanosis'),
        ('2', 'no cyanosis'),
        ], 'Appearance', required=True)

    apgar_pulse = fields.Selection([
        ('0', 'Absent'),
        ('1', '< 100'),
        ('2', '> 100'),
        ], 'Pulse', required=True)

    apgar_grimace = fields.Selection([
        ('0', 'No response to stimulation'),
        ('1', 'grimace when stimulated'),
        ('2', 'cry or pull away when stimulated'),
        ], 'Grimace', required=True, sort=False)

    apgar_activity = fields.Selection([
        ('0', 'None'),
        ('1', 'Some flexion'),
        ('2', 'flexed arms and legs'),
        ], 'Activity', required=True, sort=False)

    apgar_respiration = fields.Selection([
        ('0', 'Absent'),
        ('1', 'Weak / Irregular'),
        ('2', 'strong'),
        ], 'Respiration', required=True, sort=False)

    apgar_score = fields.Integer('APGAR Score')

    @fields.depends('apgar_respiration', 'apgar_activity', 'apgar_grimace',
        'apgar_pulse', 'apgar_appearance')
    def on_change_with_apgar_score(self):
        apgar_appearance = self.apgar_appearance or '0'
        apgar_pulse = self.apgar_pulse or '0'
        apgar_grimace = self.apgar_grimace or '0'
        apgar_activity = self.apgar_activity or '0'
        apgar_respiration = self.apgar_respiration or '0'

        apgar_score = int(apgar_appearance) + int(apgar_pulse) + \
            int(apgar_grimace) + int(apgar_activity) + int(apgar_respiration)

        return apgar_score


# Deprecated in 3.0 - Use the main patient form  
class NeonatalMedication(ModelSQL, ModelView):
    'Neonatal Medication. Inherit and Add field to Medication model'
    __name__ = 'gnuhealth.patient.medication'

    newborn_id = fields.Many2One('gnuhealth.newborn', 'Newborn ID')

#Deprecated in 3.0  - Use main patient form
class NeonatalCongenitalDiseases(ModelSQL, ModelView):
    'Congenital Diseases. Inherit Disease object for use in neonatology'
    __name__ = 'gnuhealth.patient.disease'

    newborn_id = fields.Many2One('gnuhealth.newborn', 'Newborn ID')


class PediatricSymptomsChecklist(ModelSQL, ModelView):
    'Pediatric Symptoms Checklist'
    __name__ = 'gnuhealth.patient.psc'

    patient = fields.Many2One('gnuhealth.patient', 'Patient', required=True)

    evaluation_date = fields.Many2One('gnuhealth.appointment', 'Appointment',
        help="Enter or select the date / ID of the appointment related to "
        "this evaluation")

    evaluation_start = fields.DateTime('Date', required=True)

    user_id = fields.Many2One('res.user', 'Health Professional', readonly=True)

    notes = fields.Text('Notes')

    psc_aches_pains = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Complains of aches and pains', sort=False)

    psc_spend_time_alone = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Spends more time alone', sort=False)

    psc_tires_easily = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Tires easily, has little energy', sort=False)

    psc_fidgety = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Fidgety, unable to sit still', sort=False)

    psc_trouble_with_teacher = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Has trouble with teacher', sort=False)

    psc_less_interest_in_school = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Less interested in school', sort=False)

    psc_acts_as_driven_by_motor = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Acts as if driven by a motor', sort=False)

    psc_daydreams_too_much = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Daydreams too much', sort=False)

    psc_distracted_easily = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Distracted easily', sort=False)

    psc_afraid_of_new_situations = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Is afraid of new situations', sort=False)

    psc_sad_unhappy = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Feels sad, unhappy', sort=False)

    psc_irritable_angry = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Is irritable, angry', sort=False)

    psc_feels_hopeless = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Feels hopeless', sort=False)

    psc_trouble_concentrating = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Has trouble concentrating', sort=False)

    psc_less_interested_in_friends = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Less interested in friends', sort=False)

    psc_fights_with_others = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Fights with other children', sort=False)

    psc_absent_from_school = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Absent from school', sort=False)

    psc_school_grades_dropping = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'School grades dropping', sort=False)

    psc_down_on_self = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Is down on him or herself', sort=False)

    psc_visit_doctor_finds_ok = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Visits the doctor with doctor finding nothing wrong', sort=False)

    psc_trouble_sleeping = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Has trouble sleeping', sort=False)

    psc_worries_a_lot = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Worries a lot', sort=False)

    psc_wants_to_be_with_parents = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Wants to be with you more than before', sort=False)

    psc_feels_is_bad_child = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Feels he or she is bad', sort=False)

    psc_takes_unnecesary_risks = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Takes unnecessary risks', sort=False)

    psc_gets_hurt_often = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Gets hurt frequently', sort=False)

    psc_having_less_fun = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Seems to be having less fun', sort=False)

    psc_act_as_younger = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Acts younger than children his or her age', sort=False)

    psc_does_not_listen_to_rules = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Does not listen to rules', sort=False)

    psc_does_not_show_feelings = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Does not show feelings', sort=False)

    psc_does_not_get_people_feelings = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Does not get people feelings', sort=False)

    psc_teases_others = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Teases others', sort=False)

    psc_blames_others = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Blames others for his or her troubles', sort=False)

    psc_takes_things_from_others = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Takes things that do not belong to him or her', sort=False)

    psc_refuses_to_share = fields.Selection([
        (None, ''),
        ('0', 'Never'),
        ('1', 'Sometimes'),
        ('2', 'Often'),
        ], 'Refuses to share', sort=False)

    psc_total = fields.Integer('PSC Total')

    @staticmethod
    def default_user_id():
        User = Pool().get('res.user')
        user = User(Transaction().user)
        return int(user.id)

    @staticmethod
    def default_psc_total():
        return 0

    @fields.depends('psc_aches_pains', 'psc_spend_time_alone',
        'psc_tires_easily', 'psc_fidgety', 'psc_trouble_with_teacher',
        'psc_less_interest_in_school', 'psc_acts_as_driven_by_motor',
        'psc_daydreams_too_much', 'psc_distracted_easily',
        'psc_afraid_of_new_situations', 'psc_sad_unhappy',
        'psc_irritable_angry', 'psc_feels_hopeless',
        'psc_trouble_concentrating', 'psc_less_interested_in_friends',
        'psc_fights_with_others', 'psc_absent_from_school',
        'psc_school_grades_dropping', 'psc_down_on_self',
        'psc_visit_doctor_finds_ok', 'psc_trouble_sleeping',
        'psc_worries_a_lot', 'psc_wants_to_be_with_parents',
        'psc_feels_is_bad_child', 'psc_takes_unnecesary_risks',
        'psc_gets_hurt_often', 'psc_having_less_fun',
        'psc_act_as_younger', 'psc_does_not_listen_to_rules',
        'psc_does_not_show_feelings',
        'psc_does_not_get_people_feelings',
        'psc_teases_others', 'psc_takes_things_from_others',
        'psc_refuses_to_share')
    def on_change_with_psc_total(self):

        psc_aches_pains = self.psc_aches_pains or '0'
        psc_spend_time_alone = self.psc_spend_time_alone or '0'
        psc_tires_easily = self.psc_tires_easily or '0'
        psc_fidgety = self.psc_fidgety or '0'
        psc_trouble_with_teacher = self.psc_trouble_with_teacher or '0'
        psc_less_interest_in_school = self.psc_less_interest_in_school or '0'
        psc_acts_as_driven_by_motor = self.psc_acts_as_driven_by_motor or '0'
        psc_daydreams_too_much = self.psc_daydreams_too_much or '0'
        psc_distracted_easily = self.psc_distracted_easily or '0'
        psc_afraid_of_new_situations = self.psc_afraid_of_new_situations or '0'
        psc_sad_unhappy = self.psc_sad_unhappy or '0'
        psc_irritable_angry = self.psc_irritable_angry or '0'
        psc_feels_hopeless = self.psc_feels_hopeless or '0'
        psc_trouble_concentrating = self.psc_trouble_concentrating or '0'
        psc_less_interested_in_friends = \
                self.psc_less_interested_in_friends or '0'
        psc_fights_with_others = self.psc_fights_with_others or '0'
        psc_absent_from_school = self.psc_absent_from_school or '0'
        psc_school_grades_dropping = self.psc_school_grades_dropping or '0'
        psc_down_on_self = self.psc_down_on_self or '0'
        psc_visit_doctor_finds_ok = self.psc_visit_doctor_finds_ok or '0'
        psc_trouble_sleeping = self.psc_trouble_sleeping or '0'
        psc_worries_a_lot = self.psc_worries_a_lot or '0'
        psc_wants_to_be_with_parents = self.psc_wants_to_be_with_parents or '0'
        psc_feels_is_bad_child = self.psc_feels_is_bad_child or '0'
        psc_takes_unnecesary_risks = self.psc_takes_unnecesary_risks or '0'
        psc_gets_hurt_often = self.psc_gets_hurt_often or '0'
        psc_having_less_fun = self.psc_having_less_fun or '0'
        psc_act_as_younger = self.psc_act_as_younger or '0'
        psc_does_not_listen_to_rules = self.psc_does_not_listen_to_rules or '0'
        psc_does_not_show_feelings = self.psc_does_not_show_feelings or '0'
        psc_does_not_get_people_feelings = \
                self.psc_does_not_get_people_feelings or '0'
        psc_teases_others = self.psc_teases_others or '0'
        psc_takes_things_from_others = self.psc_takes_things_from_others or '0'
        psc_refuses_to_share = self.psc_refuses_to_share or '0'

        psc_total = int(psc_aches_pains) + int(psc_spend_time_alone) + \
            int(psc_tires_easily) + int(psc_fidgety) + \
            int(psc_trouble_with_teacher) + \
            int(psc_less_interest_in_school) + \
            int(psc_acts_as_driven_by_motor) + \
            int(psc_daydreams_too_much) + int(psc_distracted_easily) + \
            int(psc_afraid_of_new_situations) + int(psc_sad_unhappy) + \
            int(psc_irritable_angry) + int(psc_feels_hopeless) + \
            int(psc_trouble_concentrating) + \
            int(psc_less_interested_in_friends) + \
            int(psc_fights_with_others) + int(psc_absent_from_school) + \
            int(psc_school_grades_dropping) + int(psc_down_on_self) + \
            int(psc_visit_doctor_finds_ok) + int(psc_trouble_sleeping) + \
            int(psc_worries_a_lot) + int(psc_wants_to_be_with_parents) + \
            int(psc_feels_is_bad_child) + int(psc_takes_unnecesary_risks) + \
            int(psc_gets_hurt_often) + int(psc_having_less_fun) + \
            int(psc_act_as_younger) + int(psc_does_not_listen_to_rules) + \
            int(psc_does_not_show_feelings) + \
            int(psc_does_not_get_people_feelings) + \
            int(psc_teases_others) + \
            int(psc_takes_things_from_others) + \
            int(psc_refuses_to_share)

        return psc_total


# REMOVED IN 1.6
# WE USE A RELATE ACTION

'''
class PscEvaluation(ModelSQL, ModelView):
    'Pediatric Symptoms Checklist Evaluation'
    _name = 'gnuhealth.patient'
    _description = __doc__

    psc = fields.One2Many('gnuhealth.patient.psc', 'patient',
        'Pediatric Symptoms Checklist')

PscEvaluation()
'''



import re as re_

ExternalEncoding = 'utf-8'
Tag_pattern_ = re_.compile(r'({.*})?(.*)')
String_cleanup_pat_ = re_.compile(r"[\n\r\s]+")
Namespace_extract_pat_ = re_.compile(r'{(.*)}(.*)')

Validate_simpletypes_ = True
etree_ = None
Verbose_import_ = False
(
    XMLParser_import_none, XMLParser_import_lxml,
    XMLParser_import_elementtree
) = list(range(3))
XMLParser_import_library = None
try:
    # lxml
    from lxml import etree as etree_
    XMLParser_import_library = XMLParser_import_lxml
    if Verbose_import_:
        print("running with lxml.etree")
except ImportError:
    try:
        # cElementTree from Python 2.5+
        import xml.etree.cElementTree as etree_
        XMLParser_import_library = XMLParser_import_elementtree
        if Verbose_import_:
            print("running with cElementTree on Python 2.5+")
    except ImportError:
        try:
            # ElementTree from Python 2.5+
            import xml.etree.ElementTree as etree_
            XMLParser_import_library = XMLParser_import_elementtree
            if Verbose_import_:
                print("running with ElementTree on Python 2.5+")
        except ImportError:
            try:
                # normal cElementTree install
                import cElementTree as etree_
                XMLParser_import_library = XMLParser_import_elementtree
                if Verbose_import_:
                    print("running with cElementTree")
            except ImportError:
                try:
                    # normal ElementTree install
                    import elementtree.ElementTree as etree_
                    XMLParser_import_library = XMLParser_import_elementtree
                    if Verbose_import_:
                        print("running with ElementTree")
                except ImportError:
                    raise ImportError(
                        "Failed to import ElementTree from any known place")


from functools import partial
import defusedxml.lxml

#### SAFE PARSERS #####
safe_parse = partial(defusedxml.lxml.parse, forbid_dtd=True,
                                forbid_entities=True)
safe_fromstring = partial(defusedxml.lxml.fromstring, forbid_dtd=True,
                                forbid_entities=True)
#### /SAFE PARSERS #####



def parsexml_(*args, **kwargs):
    if (XMLParser_import_library == XMLParser_import_lxml and
            'parser' not in kwargs):
        # Use the lxml ElementTree compatible parser so that, e.g.,
        #   we ignore comments.
        kwargs['parser'] = etree_.ETCompatXMLParser()
    #doc = etree_.parse(*args, **kwargs)
    doc = safe_parse(*args, **kwargs)
    return doc



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from trytond.model import ModelView
from trytond.wizard import Wizard, StateTransition, StateAction, StateView, Button
from trytond.transaction import Transaction
from trytond.pool import Pool
from trytond.pyson import PYSONEncoder


__all__ = ['CreateAppointmentEvaluation']

class CreateAppointmentEvaluation(Wizard):
    'Create Appointment Evaluation'
    __name__ = 'wizard.gnuhealth.appointment.evaluation'
  
    start_state = 'appointment_evaluation'
    appointment_evaluation = StateAction('health.act_app_evaluation')

    def do_appointment_evaluation(self, action):
      
        appointment = Transaction().context.get('active_id')

        try:
            app_id = \
                Pool().get('gnuhealth.appointment').browse([appointment])[0]
        except:
            self.raise_user_error('no_record_selected')
            
        patient = app_id.patient.id

        if (app_id.speciality):
            specialty = app_id.speciality.id
        else:
            specialty = None
        urgency = str(app_id.urgency)
        evaluation_type = str(app_id.appointment_type)
        visit_type = str(app_id.visit_type)

        action['pyson_domain'] = PYSONEncoder().encode([
            ('appointment', '=', appointment),
            ('patient', '=', patient),
            ('specialty', '=', specialty),
            ('urgency', '=', urgency),
            ('evaluation_type', '=', evaluation_type),
            ('visit_type', '=', visit_type),
            ])
        action['pyson_context'] = PYSONEncoder().encode({
            'appointment': appointment,
            'patient': patient,
            'specialty': specialty,
            'urgency': urgency,
            'evaluation_type': evaluation_type,
            'visit_type': visit_type,
            })
            
        return action, {}
        
    @classmethod
    def __setup__(cls):
        super(CreateAppointmentEvaluation, cls).__setup__()
        cls._error_messages.update({
            'no_record_selected':
                'You need to select one Appointment record',
        })




from .base_classes import *
from .support_functions import *


class Encounter(Resource):
    """An interaction between a patient and healthcare provider(s) for the
    purpose of providing healthcare service(s) or assessing the
    health status of a patient.If the element is present, it must
    have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, status=None, class_=None, type_=None, subject=None, participant=None, period=None, length=None, reason=None, indication=None, priority=None, hospitalization=None, location=None, serviceProvider=None, partOf=None):
        self.original_tagname_ = None
        super(Encounter, self).__init__(id, extension, modifierExtension, language, text, contained, )
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.status = status
        self.class_ = class_
        if type_ is None:
            self.type_ = []
        else:
            self.type_ = type_
        self.subject = subject
        if participant is None:
            self.participant = []
        else:
            self.participant = participant
        self.period = period
        self.length = length
        self.reason = reason
        self.indication = indication
        self.priority = priority
        self.hospitalization = hospitalization
        if location is None:
            self.location = []
        else:
            self.location = location
        self.serviceProvider = serviceProvider
        self.partOf = partOf
    def factory(*args_, **kwargs_):
        if Encounter.subclass:
            return Encounter.subclass(*args_, **kwargs_)
        else:
            return Encounter(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_status(self): return self.status
    def set_status(self, status): self.status = status
    def get_class(self): return self.class_
    def set_class(self, class_): self.class_ = class_
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def add_type(self, value): self.type_.append(value)
    def insert_type(self, index, value): self.type_[index] = value
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_participant(self): return self.participant
    def set_participant(self, participant): self.participant = participant
    def add_participant(self, value): self.participant.append(value)
    def insert_participant(self, index, value): self.participant[index] = value
    def get_period(self): return self.period
    def set_period(self, period): self.period = period
    def get_length(self): return self.length
    def set_length(self, length): self.length = length
    def get_reason(self): return self.reason
    def set_reason(self, reason): self.reason = reason
    def get_indication(self): return self.indication
    def set_indication(self, indication): self.indication = indication
    def get_priority(self): return self.priority
    def set_priority(self, priority): self.priority = priority
    def get_hospitalization(self): return self.hospitalization
    def set_hospitalization(self, hospitalization): self.hospitalization = hospitalization
    def get_location(self): return self.location
    def set_location(self, location): self.location = location
    def add_location(self, value): self.location.append(value)
    def insert_location(self, index, value): self.location[index] = value
    def get_serviceProvider(self): return self.serviceProvider
    def set_serviceProvider(self, serviceProvider): self.serviceProvider = serviceProvider
    def get_partOf(self): return self.partOf
    def set_partOf(self, partOf): self.partOf = partOf
    def hasContent_(self):
        if (
            self.identifier or
            self.status is not None or
            self.class_ is not None or
            self.type_ or
            self.subject is not None or
            self.participant or
            self.period is not None or
            self.length is not None or
            self.reason is not None or
            self.indication is not None or
            self.priority is not None or
            self.hospitalization is not None or
            self.location or
            self.serviceProvider is not None or
            self.partOf is not None or
            super(Encounter, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Encounter', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Encounter')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Encounter', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Encounter'):
        super(Encounter, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Encounter')
    def exportChildren(self, outfile, level, namespace_='', name_='Encounter', fromsubclass_=False, pretty_print=True):
        super(Encounter, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.status is not None:
            self.status.export(outfile, level, namespace_, name_='status', pretty_print=pretty_print)
        if self.class_ is not None:
            self.class_.export(outfile, level, namespace_, name_='class', pretty_print=pretty_print)
        for type_ in self.type_:
            type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        for participant_ in self.participant:
            participant_.export(outfile, level, namespace_, name_='participant', pretty_print=pretty_print)
        if self.period is not None:
            self.period.export(outfile, level, namespace_, name_='period', pretty_print=pretty_print)
        if self.length is not None:
            self.length.export(outfile, level, namespace_, name_='length', pretty_print=pretty_print)
        if self.reason is not None:
            self.reason.export(outfile, level, namespace_, name_='reason', pretty_print=pretty_print)
        if self.indication is not None:
            self.indication.export(outfile, level, namespace_, name_='indication', pretty_print=pretty_print)
        if self.priority is not None:
            self.priority.export(outfile, level, namespace_, name_='priority', pretty_print=pretty_print)
        if self.hospitalization is not None:
            self.hospitalization.export(outfile, level, namespace_, name_='hospitalization', pretty_print=pretty_print)
        for location_ in self.location:
            location_.export(outfile, level, namespace_, name_='location', pretty_print=pretty_print)
        if self.serviceProvider is not None:
            self.serviceProvider.export(outfile, level, namespace_, name_='serviceProvider', pretty_print=pretty_print)
        if self.partOf is not None:
            self.partOf.export(outfile, level, namespace_, name_='partOf', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Encounter', mapping_=None):
        element = super(Encounter, self).to_etree(parent_element, name_, mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.status is not None:
            status_ = self.status
            status_.to_etree(element, name_='status', mapping_=mapping_)
        if self.class_ is not None:
            class__ = self.class_
            class__.to_etree(element, name_='class', mapping_=mapping_)
        for type__ in self.type_:
            type__.to_etree(element, name_='type', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        for participant_ in self.participant:
            participant_.to_etree(element, name_='participant', mapping_=mapping_)
        if self.period is not None:
            period_ = self.period
            period_.to_etree(element, name_='period', mapping_=mapping_)
        if self.length is not None:
            length_ = self.length
            length_.to_etree(element, name_='length', mapping_=mapping_)
        if self.reason is not None:
            reason_ = self.reason
            reason_.to_etree(element, name_='reason', mapping_=mapping_)
        if self.indication is not None:
            indication_ = self.indication
            indication_.to_etree(element, name_='indication', mapping_=mapping_)
        if self.priority is not None:
            priority_ = self.priority
            priority_.to_etree(element, name_='priority', mapping_=mapping_)
        if self.hospitalization is not None:
            hospitalization_ = self.hospitalization
            hospitalization_.to_etree(element, name_='hospitalization', mapping_=mapping_)
        for location_ in self.location:
            location_.to_etree(element, name_='location', mapping_=mapping_)
        if self.serviceProvider is not None:
            serviceProvider_ = self.serviceProvider
            serviceProvider_.to_etree(element, name_='serviceProvider', mapping_=mapping_)
        if self.partOf is not None:
            partOf_ = self.partOf
            partOf_.to_etree(element, name_='partOf', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Encounter, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'status':
            obj_ = EncounterState.factory()
            obj_.build(child_)
            self.status = obj_
            obj_.original_tagname_ = 'status'
        elif nodeName_ == 'class':
            obj_ = EncounterClass.factory()
            obj_.build(child_)
            self.class_ = obj_
            obj_.original_tagname_ = 'class'
        elif nodeName_ == 'type':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.type_.append(obj_)
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'participant':
            obj_ = Encounter_Participant.factory()
            obj_.build(child_)
            self.participant.append(obj_)
            obj_.original_tagname_ = 'participant'
        elif nodeName_ == 'period':
            obj_ = Period.factory()
            obj_.build(child_)
            self.period = obj_
            obj_.original_tagname_ = 'period'
        elif nodeName_ == 'length':
            obj_ = Duration.factory()
            obj_.build(child_)
            self.length = obj_
            obj_.original_tagname_ = 'length'
        elif nodeName_ == 'reason':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.reason = obj_
            obj_.original_tagname_ = 'reason'
        elif nodeName_ == 'indication':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.indication = obj_
            obj_.original_tagname_ = 'indication'
        elif nodeName_ == 'priority':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.priority = obj_
            obj_.original_tagname_ = 'priority'
        elif nodeName_ == 'hospitalization':
            obj_ = Encounter_Hospitalization.factory()
            obj_.build(child_)
            self.hospitalization = obj_
            obj_.original_tagname_ = 'hospitalization'
        elif nodeName_ == 'location':
            obj_ = Encounter_Location.factory()
            obj_.build(child_)
            self.location.append(obj_)
            obj_.original_tagname_ = 'location'
        elif nodeName_ == 'serviceProvider':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.serviceProvider = obj_
            obj_.original_tagname_ = 'serviceProvider'
        elif nodeName_ == 'partOf':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.partOf = obj_
            obj_.original_tagname_ = 'partOf'
        super(Encounter, self).buildChildren(child_, node, nodeName_, True)
# end class Encounter


class Encounter_Participant(BackboneElement):
    """An interaction between a patient and healthcare provider(s) for the
    purpose of providing healthcare service(s) or assessing the
    health status of a patient."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, type_=None, individual=None):
        self.original_tagname_ = None
        super(Encounter_Participant, self).__init__(id, extension, modifierExtension, )
        if type_ is None:
            self.type_ = []
        else:
            self.type_ = type_
        self.individual = individual
    def factory(*args_, **kwargs_):
        if Encounter_Participant.subclass:
            return Encounter_Participant.subclass(*args_, **kwargs_)
        else:
            return Encounter_Participant(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def add_type(self, value): self.type_.append(value)
    def insert_type(self, index, value): self.type_[index] = value
    def get_individual(self): return self.individual
    def set_individual(self, individual): self.individual = individual
    def hasContent_(self):
        if (
            self.type_ or
            self.individual is not None or
            super(Encounter_Participant, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Encounter.Participant', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Encounter.Participant')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Encounter.Participant', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Encounter.Participant'):
        super(Encounter_Participant, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Encounter.Participant')
    def exportChildren(self, outfile, level, namespace_='', name_='Encounter.Participant', fromsubclass_=False, pretty_print=True):
        super(Encounter_Participant, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for type_ in self.type_:
            type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        if self.individual is not None:
            self.individual.export(outfile, level, namespace_, name_='individual', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Encounter.Participant', mapping_=None):
        element = super(Encounter_Participant, self).to_etree(parent_element, name_, mapping_)
        for type__ in self.type_:
            type__.to_etree(element, name_='type', mapping_=mapping_)
        if self.individual is not None:
            individual_ = self.individual
            individual_.to_etree(element, name_='individual', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Encounter_Participant, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'type':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.type_.append(obj_)
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'individual':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.individual = obj_
            obj_.original_tagname_ = 'individual'
        super(Encounter_Participant, self).buildChildren(child_, node, nodeName_, True)
# end class Encounter_Participant


class Encounter_Hospitalization(BackboneElement):
    """An interaction between a patient and healthcare provider(s) for the
    purpose of providing healthcare service(s) or assessing the
    health status of a patient."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, preAdmissionIdentifier=None, origin=None, admitSource=None, period=None, accomodation=None, diet=None, specialCourtesy=None, specialArrangement=None, destination=None, dischargeDisposition=None, dischargeDiagnosis=None, reAdmission=None):
        self.original_tagname_ = None
        super(Encounter_Hospitalization, self).__init__(id, extension, modifierExtension, )
        self.preAdmissionIdentifier = preAdmissionIdentifier
        self.origin = origin
        self.admitSource = admitSource
        self.period = period
        if accomodation is None:
            self.accomodation = []
        else:
            self.accomodation = accomodation
        self.diet = diet
        if specialCourtesy is None:
            self.specialCourtesy = []
        else:
            self.specialCourtesy = specialCourtesy
        if specialArrangement is None:
            self.specialArrangement = []
        else:
            self.specialArrangement = specialArrangement
        self.destination = destination
        self.dischargeDisposition = dischargeDisposition
        self.dischargeDiagnosis = dischargeDiagnosis
        self.reAdmission = reAdmission
    def factory(*args_, **kwargs_):
        if Encounter_Hospitalization.subclass:
            return Encounter_Hospitalization.subclass(*args_, **kwargs_)
        else:
            return Encounter_Hospitalization(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_preAdmissionIdentifier(self): return self.preAdmissionIdentifier
    def set_preAdmissionIdentifier(self, preAdmissionIdentifier): self.preAdmissionIdentifier = preAdmissionIdentifier
    def get_origin(self): return self.origin
    def set_origin(self, origin): self.origin = origin
    def get_admitSource(self): return self.admitSource
    def set_admitSource(self, admitSource): self.admitSource = admitSource
    def get_period(self): return self.period
    def set_period(self, period): self.period = period
    def get_accomodation(self): return self.accomodation
    def set_accomodation(self, accomodation): self.accomodation = accomodation
    def add_accomodation(self, value): self.accomodation.append(value)
    def insert_accomodation(self, index, value): self.accomodation[index] = value
    def get_diet(self): return self.diet
    def set_diet(self, diet): self.diet = diet
    def get_specialCourtesy(self): return self.specialCourtesy
    def set_specialCourtesy(self, specialCourtesy): self.specialCourtesy = specialCourtesy
    def add_specialCourtesy(self, value): self.specialCourtesy.append(value)
    def insert_specialCourtesy(self, index, value): self.specialCourtesy[index] = value
    def get_specialArrangement(self): return self.specialArrangement
    def set_specialArrangement(self, specialArrangement): self.specialArrangement = specialArrangement
    def add_specialArrangement(self, value): self.specialArrangement.append(value)
    def insert_specialArrangement(self, index, value): self.specialArrangement[index] = value
    def get_destination(self): return self.destination
    def set_destination(self, destination): self.destination = destination
    def get_dischargeDisposition(self): return self.dischargeDisposition
    def set_dischargeDisposition(self, dischargeDisposition): self.dischargeDisposition = dischargeDisposition
    def get_dischargeDiagnosis(self): return self.dischargeDiagnosis
    def set_dischargeDiagnosis(self, dischargeDiagnosis): self.dischargeDiagnosis = dischargeDiagnosis
    def get_reAdmission(self): return self.reAdmission
    def set_reAdmission(self, reAdmission): self.reAdmission = reAdmission
    def hasContent_(self):
        if (
            self.preAdmissionIdentifier is not None or
            self.origin is not None or
            self.admitSource is not None or
            self.period is not None or
            self.accomodation or
            self.diet is not None or
            self.specialCourtesy or
            self.specialArrangement or
            self.destination is not None or
            self.dischargeDisposition is not None or
            self.dischargeDiagnosis is not None or
            self.reAdmission is not None or
            super(Encounter_Hospitalization, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Encounter.Hospitalization', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Encounter.Hospitalization')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Encounter.Hospitalization', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Encounter.Hospitalization'):
        super(Encounter_Hospitalization, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Encounter.Hospitalization')
    def exportChildren(self, outfile, level, namespace_='', name_='Encounter.Hospitalization', fromsubclass_=False, pretty_print=True):
        super(Encounter_Hospitalization, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.preAdmissionIdentifier is not None:
            self.preAdmissionIdentifier.export(outfile, level, namespace_, name_='preAdmissionIdentifier', pretty_print=pretty_print)
        if self.origin is not None:
            self.origin.export(outfile, level, namespace_, name_='origin', pretty_print=pretty_print)
        if self.admitSource is not None:
            self.admitSource.export(outfile, level, namespace_, name_='admitSource', pretty_print=pretty_print)
        if self.period is not None:
            self.period.export(outfile, level, namespace_, name_='period', pretty_print=pretty_print)
        for accomodation_ in self.accomodation:
            accomodation_.export(outfile, level, namespace_, name_='accomodation', pretty_print=pretty_print)
        if self.diet is not None:
            self.diet.export(outfile, level, namespace_, name_='diet', pretty_print=pretty_print)
        for specialCourtesy_ in self.specialCourtesy:
            specialCourtesy_.export(outfile, level, namespace_, name_='specialCourtesy', pretty_print=pretty_print)
        for specialArrangement_ in self.specialArrangement:
            specialArrangement_.export(outfile, level, namespace_, name_='specialArrangement', pretty_print=pretty_print)
        if self.destination is not None:
            self.destination.export(outfile, level, namespace_, name_='destination', pretty_print=pretty_print)
        if self.dischargeDisposition is not None:
            self.dischargeDisposition.export(outfile, level, namespace_, name_='dischargeDisposition', pretty_print=pretty_print)
        if self.dischargeDiagnosis is not None:
            self.dischargeDiagnosis.export(outfile, level, namespace_, name_='dischargeDiagnosis', pretty_print=pretty_print)
        if self.reAdmission is not None:
            self.reAdmission.export(outfile, level, namespace_, name_='reAdmission', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Encounter.Hospitalization', mapping_=None):
        element = super(Encounter_Hospitalization, self).to_etree(parent_element, name_, mapping_)
        if self.preAdmissionIdentifier is not None:
            preAdmissionIdentifier_ = self.preAdmissionIdentifier
            preAdmissionIdentifier_.to_etree(element, name_='preAdmissionIdentifier', mapping_=mapping_)
        if self.origin is not None:
            origin_ = self.origin
            origin_.to_etree(element, name_='origin', mapping_=mapping_)
        if self.admitSource is not None:
            admitSource_ = self.admitSource
            admitSource_.to_etree(element, name_='admitSource', mapping_=mapping_)
        if self.period is not None:
            period_ = self.period
            period_.to_etree(element, name_='period', mapping_=mapping_)
        for accomodation_ in self.accomodation:
            accomodation_.to_etree(element, name_='accomodation', mapping_=mapping_)
        if self.diet is not None:
            diet_ = self.diet
            diet_.to_etree(element, name_='diet', mapping_=mapping_)
        for specialCourtesy_ in self.specialCourtesy:
            specialCourtesy_.to_etree(element, name_='specialCourtesy', mapping_=mapping_)
        for specialArrangement_ in self.specialArrangement:
            specialArrangement_.to_etree(element, name_='specialArrangement', mapping_=mapping_)
        if self.destination is not None:
            destination_ = self.destination
            destination_.to_etree(element, name_='destination', mapping_=mapping_)
        if self.dischargeDisposition is not None:
            dischargeDisposition_ = self.dischargeDisposition
            dischargeDisposition_.to_etree(element, name_='dischargeDisposition', mapping_=mapping_)
        if self.dischargeDiagnosis is not None:
            dischargeDiagnosis_ = self.dischargeDiagnosis
            dischargeDiagnosis_.to_etree(element, name_='dischargeDiagnosis', mapping_=mapping_)
        if self.reAdmission is not None:
            reAdmission_ = self.reAdmission
            reAdmission_.to_etree(element, name_='reAdmission', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Encounter_Hospitalization, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'preAdmissionIdentifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.preAdmissionIdentifier = obj_
            obj_.original_tagname_ = 'preAdmissionIdentifier'
        elif nodeName_ == 'origin':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.origin = obj_
            obj_.original_tagname_ = 'origin'
        elif nodeName_ == 'admitSource':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.admitSource = obj_
            obj_.original_tagname_ = 'admitSource'
        elif nodeName_ == 'period':
            obj_ = Period.factory()
            obj_.build(child_)
            self.period = obj_
            obj_.original_tagname_ = 'period'
        elif nodeName_ == 'accomodation':
            obj_ = Encounter_Accomodation.factory()
            obj_.build(child_)
            self.accomodation.append(obj_)
            obj_.original_tagname_ = 'accomodation'
        elif nodeName_ == 'diet':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.diet = obj_
            obj_.original_tagname_ = 'diet'
        elif nodeName_ == 'specialCourtesy':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.specialCourtesy.append(obj_)
            obj_.original_tagname_ = 'specialCourtesy'
        elif nodeName_ == 'specialArrangement':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.specialArrangement.append(obj_)
            obj_.original_tagname_ = 'specialArrangement'
        elif nodeName_ == 'destination':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.destination = obj_
            obj_.original_tagname_ = 'destination'
        elif nodeName_ == 'dischargeDisposition':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.dischargeDisposition = obj_
            obj_.original_tagname_ = 'dischargeDisposition'
        elif nodeName_ == 'dischargeDiagnosis':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.dischargeDiagnosis = obj_
            obj_.original_tagname_ = 'dischargeDiagnosis'
        elif nodeName_ == 'reAdmission':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.reAdmission = obj_
            obj_.original_tagname_ = 'reAdmission'
        super(Encounter_Hospitalization, self).buildChildren(child_, node, nodeName_, True)
# end class Encounter_Hospitalization


class Encounter_Accomodation(BackboneElement):
    """An interaction between a patient and healthcare provider(s) for the
    purpose of providing healthcare service(s) or assessing the
    health status of a patient."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, bed=None, period=None):
        self.original_tagname_ = None
        super(Encounter_Accomodation, self).__init__(id, extension, modifierExtension, )
        self.bed = bed
        self.period = period
    def factory(*args_, **kwargs_):
        if Encounter_Accomodation.subclass:
            return Encounter_Accomodation.subclass(*args_, **kwargs_)
        else:
            return Encounter_Accomodation(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_bed(self): return self.bed
    def set_bed(self, bed): self.bed = bed
    def get_period(self): return self.period
    def set_period(self, period): self.period = period
    def hasContent_(self):
        if (
            self.bed is not None or
            self.period is not None or
            super(Encounter_Accomodation, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Encounter.Accomodation', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Encounter.Accomodation')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Encounter.Accomodation', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Encounter.Accomodation'):
        super(Encounter_Accomodation, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Encounter.Accomodation')
    def exportChildren(self, outfile, level, namespace_='', name_='Encounter.Accomodation', fromsubclass_=False, pretty_print=True):
        super(Encounter_Accomodation, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.bed is not None:
            self.bed.export(outfile, level, namespace_, name_='bed', pretty_print=pretty_print)
        if self.period is not None:
            self.period.export(outfile, level, namespace_, name_='period', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Encounter.Accomodation', mapping_=None):
        element = super(Encounter_Accomodation, self).to_etree(parent_element, name_, mapping_)
        if self.bed is not None:
            bed_ = self.bed
            bed_.to_etree(element, name_='bed', mapping_=mapping_)
        if self.period is not None:
            period_ = self.period
            period_.to_etree(element, name_='period', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Encounter_Accomodation, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'bed':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.bed = obj_
            obj_.original_tagname_ = 'bed'
        elif nodeName_ == 'period':
            obj_ = Period.factory()
            obj_.build(child_)
            self.period = obj_
            obj_.original_tagname_ = 'period'
        super(Encounter_Accomodation, self).buildChildren(child_, node, nodeName_, True)
# end class Encounter_Accomodation


class Encounter_Location(BackboneElement):
    """An interaction between a patient and healthcare provider(s) for the
    purpose of providing healthcare service(s) or assessing the
    health status of a patient."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, location=None, period=None):
        self.original_tagname_ = None
        super(Encounter_Location, self).__init__(id, extension, modifierExtension, )
        self.location = location
        self.period = period
    def factory(*args_, **kwargs_):
        if Encounter_Location.subclass:
            return Encounter_Location.subclass(*args_, **kwargs_)
        else:
            return Encounter_Location(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_location(self): return self.location
    def set_location(self, location): self.location = location
    def get_period(self): return self.period
    def set_period(self, period): self.period = period
    def hasContent_(self):
        if (
            self.location is not None or
            self.period is not None or
            super(Encounter_Location, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Encounter.Location', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Encounter.Location')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Encounter.Location', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Encounter.Location'):
        super(Encounter_Location, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Encounter.Location')
    def exportChildren(self, outfile, level, namespace_='', name_='Encounter.Location', fromsubclass_=False, pretty_print=True):
        super(Encounter_Location, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.location is not None:
            self.location.export(outfile, level, namespace_, name_='location', pretty_print=pretty_print)
        if self.period is not None:
            self.period.export(outfile, level, namespace_, name_='period', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Encounter.Location', mapping_=None):
        element = super(Encounter_Location, self).to_etree(parent_element, name_, mapping_)
        if self.location is not None:
            location_ = self.location
            location_.to_etree(element, name_='location', mapping_=mapping_)
        if self.period is not None:
            period_ = self.period
            period_.to_etree(element, name_='period', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Encounter_Location, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'location':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.location = obj_
            obj_.original_tagname_ = 'location'
        elif nodeName_ == 'period':
            obj_ = Period.factory()
            obj_.build(child_)
            self.period = obj_
            obj_.original_tagname_ = 'period'
        super(Encounter_Location, self).buildChildren(child_, node, nodeName_, True)
# end class Encounter_Location


class EncounterClass(Element):
    """Classification of the encounterIf the element is present, it must
    have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(EncounterClass, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if EncounterClass.subclass:
            return EncounterClass.subclass(*args_, **kwargs_)
        else:
            return EncounterClass(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(EncounterClass, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='EncounterClass', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='EncounterClass')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='EncounterClass', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='EncounterClass'):
        super(EncounterClass, self).exportAttributes(outfile, level, already_processed, namespace_, name_='EncounterClass')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='EncounterClass', fromsubclass_=False, pretty_print=True):
        super(EncounterClass, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='EncounterClass', mapping_=None):
        element = super(EncounterClass, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(EncounterClass, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(EncounterClass, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class EncounterClass


class EncounterState(Element):
    """Current state of the encounterIf the element is present, it must
    have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(EncounterState, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if EncounterState.subclass:
            return EncounterState.subclass(*args_, **kwargs_)
        else:
            return EncounterState(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(EncounterState, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='EncounterState', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='EncounterState')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='EncounterState', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='EncounterState'):
        super(EncounterState, self).exportAttributes(outfile, level, already_processed, namespace_, name_='EncounterState')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='EncounterState', fromsubclass_=False, pretty_print=True):
        super(EncounterState, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='EncounterState', mapping_=None):
        element = super(EncounterState, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(EncounterState, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(EncounterState, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class EncounterState



from .base_classes import *
from .support_functions import *


class Group(Resource):
    """Represents a defined collection of entities that may be discussed or
    acted upon collectively but which are not expected to act
    collectively and are not formally or legally recognized. I.e. A
    collection of entities that isn't an Organization.If the element
    is present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, type_=None, actual=None, code=None, name=None, quantity=None, characteristic=None, member=None):
        self.original_tagname_ = None
        super(Group, self).__init__(id, extension, modifierExtension, language, text, contained, )
        self.identifier = identifier
        self.type_ = type_
        self.actual = actual
        self.code = code
        self.name = name
        self.quantity = quantity
        if characteristic is None:
            self.characteristic = []
        else:
            self.characteristic = characteristic
        if member is None:
            self.member = []
        else:
            self.member = member
    def factory(*args_, **kwargs_):
        if Group.subclass:
            return Group.subclass(*args_, **kwargs_)
        else:
            return Group(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def get_actual(self): return self.actual
    def set_actual(self, actual): self.actual = actual
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def get_name(self): return self.name
    def set_name(self, name): self.name = name
    def get_quantity(self): return self.quantity
    def set_quantity(self, quantity): self.quantity = quantity
    def get_characteristic(self): return self.characteristic
    def set_characteristic(self, characteristic): self.characteristic = characteristic
    def add_characteristic(self, value): self.characteristic.append(value)
    def insert_characteristic(self, index, value): self.characteristic[index] = value
    def get_member(self): return self.member
    def set_member(self, member): self.member = member
    def add_member(self, value): self.member.append(value)
    def insert_member(self, index, value): self.member[index] = value
    def hasContent_(self):
        if (
            self.identifier is not None or
            self.type_ is not None or
            self.actual is not None or
            self.code is not None or
            self.name is not None or
            self.quantity is not None or
            self.characteristic or
            self.member or
            super(Group, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Group', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Group')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Group', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Group'):
        super(Group, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Group')
    def exportChildren(self, outfile, level, namespace_='', name_='Group', fromsubclass_=False, pretty_print=True):
        super(Group, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.identifier is not None:
            self.identifier.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        if self.actual is not None:
            self.actual.export(outfile, level, namespace_, name_='actual', pretty_print=pretty_print)
        if self.code is not None:
            self.code.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
        if self.name is not None:
            self.name.export(outfile, level, namespace_, name_='name', pretty_print=pretty_print)
        if self.quantity is not None:
            self.quantity.export(outfile, level, namespace_, name_='quantity', pretty_print=pretty_print)
        for characteristic_ in self.characteristic:
            characteristic_.export(outfile, level, namespace_, name_='characteristic', pretty_print=pretty_print)
        for member_ in self.member:
            member_.export(outfile, level, namespace_, name_='member', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Group', mapping_=None):
        element = super(Group, self).to_etree(parent_element, name_, mapping_)
        if self.identifier is not None:
            identifier_ = self.identifier
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        if self.actual is not None:
            actual_ = self.actual
            actual_.to_etree(element, name_='actual', mapping_=mapping_)
        if self.code is not None:
            code_ = self.code
            code_.to_etree(element, name_='code', mapping_=mapping_)
        if self.name is not None:
            name_ = self.name
            name_.to_etree(element, name_='name', mapping_=mapping_)
        if self.quantity is not None:
            quantity_ = self.quantity
            quantity_.to_etree(element, name_='quantity', mapping_=mapping_)
        for characteristic_ in self.characteristic:
            characteristic_.to_etree(element, name_='characteristic', mapping_=mapping_)
        for member_ in self.member:
            member_.to_etree(element, name_='member', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Group, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier = obj_
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'type':
            obj_ = GroupType.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'actual':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.actual = obj_
            obj_.original_tagname_ = 'actual'
        elif nodeName_ == 'code':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.code = obj_
            obj_.original_tagname_ = 'code'
        elif nodeName_ == 'name':
            obj_ = string.factory()
            obj_.build(child_)
            self.name = obj_
            obj_.original_tagname_ = 'name'
        elif nodeName_ == 'quantity':
            obj_ = integer.factory()
            obj_.build(child_)
            self.quantity = obj_
            obj_.original_tagname_ = 'quantity'
        elif nodeName_ == 'characteristic':
            obj_ = Group_Characteristic.factory()
            obj_.build(child_)
            self.characteristic.append(obj_)
            obj_.original_tagname_ = 'characteristic'
        elif nodeName_ == 'member':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.member.append(obj_)
            obj_.original_tagname_ = 'member'
        super(Group, self).buildChildren(child_, node, nodeName_, True)
# end class Group


class Group_Characteristic(BackboneElement):
    """Represents a defined collection of entities that may be discussed or
    acted upon collectively but which are not expected to act
    collectively and are not formally or legally recognized. I.e. A
    collection of entities that isn't an Organization.The value of
    the trait that holds (or does not hold - see 'exclude') for
    members of the group."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, code=None, valueCodeableConcept=None, valueBoolean=None, valueQuantity=None, valueRange=None, exclude=None):
        self.original_tagname_ = None
        super(Group_Characteristic, self).__init__(id, extension, modifierExtension, )
        self.code = code
        self.valueCodeableConcept = valueCodeableConcept
        self.valueBoolean = valueBoolean
        self.valueQuantity = valueQuantity
        self.valueRange = valueRange
        self.exclude = exclude
    def factory(*args_, **kwargs_):
        if Group_Characteristic.subclass:
            return Group_Characteristic.subclass(*args_, **kwargs_)
        else:
            return Group_Characteristic(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def get_valueCodeableConcept(self): return self.valueCodeableConcept
    def set_valueCodeableConcept(self, valueCodeableConcept): self.valueCodeableConcept = valueCodeableConcept
    def get_valueBoolean(self): return self.valueBoolean
    def set_valueBoolean(self, valueBoolean): self.valueBoolean = valueBoolean
    def get_valueQuantity(self): return self.valueQuantity
    def set_valueQuantity(self, valueQuantity): self.valueQuantity = valueQuantity
    def get_valueRange(self): return self.valueRange
    def set_valueRange(self, valueRange): self.valueRange = valueRange
    def get_exclude(self): return self.exclude
    def set_exclude(self, exclude): self.exclude = exclude
    def hasContent_(self):
        if (
            self.code is not None or
            self.valueCodeableConcept is not None or
            self.valueBoolean is not None or
            self.valueQuantity is not None or
            self.valueRange is not None or
            self.exclude is not None or
            super(Group_Characteristic, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Group.Characteristic', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Group.Characteristic')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Group.Characteristic', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Group.Characteristic'):
        super(Group_Characteristic, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Group.Characteristic')
    def exportChildren(self, outfile, level, namespace_='', name_='Group.Characteristic', fromsubclass_=False, pretty_print=True):
        super(Group_Characteristic, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.code is not None:
            self.code.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
        if self.valueCodeableConcept is not None:
            self.valueCodeableConcept.export(outfile, level, namespace_, name_='valueCodeableConcept', pretty_print=pretty_print)
        if self.valueBoolean is not None:
            self.valueBoolean.export(outfile, level, namespace_, name_='valueBoolean', pretty_print=pretty_print)
        if self.valueQuantity is not None:
            self.valueQuantity.export(outfile, level, namespace_, name_='valueQuantity', pretty_print=pretty_print)
        if self.valueRange is not None:
            self.valueRange.export(outfile, level, namespace_, name_='valueRange', pretty_print=pretty_print)
        if self.exclude is not None:
            self.exclude.export(outfile, level, namespace_, name_='exclude', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Group.Characteristic', mapping_=None):
        element = super(Group_Characteristic, self).to_etree(parent_element, name_, mapping_)
        if self.code is not None:
            code_ = self.code
            code_.to_etree(element, name_='code', mapping_=mapping_)
        if self.valueCodeableConcept is not None:
            valueCodeableConcept_ = self.valueCodeableConcept
            valueCodeableConcept_.to_etree(element, name_='valueCodeableConcept', mapping_=mapping_)
        if self.valueBoolean is not None:
            valueBoolean_ = self.valueBoolean
            valueBoolean_.to_etree(element, name_='valueBoolean', mapping_=mapping_)
        if self.valueQuantity is not None:
            valueQuantity_ = self.valueQuantity
            valueQuantity_.to_etree(element, name_='valueQuantity', mapping_=mapping_)
        if self.valueRange is not None:
            valueRange_ = self.valueRange
            valueRange_.to_etree(element, name_='valueRange', mapping_=mapping_)
        if self.exclude is not None:
            exclude_ = self.exclude
            exclude_.to_etree(element, name_='exclude', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Group_Characteristic, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'code':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.code = obj_
            obj_.original_tagname_ = 'code'
        elif nodeName_ == 'valueCodeableConcept':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.valueCodeableConcept = obj_
            obj_.original_tagname_ = 'valueCodeableConcept'
        elif nodeName_ == 'valueBoolean':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.valueBoolean = obj_
            obj_.original_tagname_ = 'valueBoolean'
        elif nodeName_ == 'valueQuantity':
            obj_ = Quantity.factory()
            obj_.build(child_)
            self.valueQuantity = obj_
            obj_.original_tagname_ = 'valueQuantity'
        elif nodeName_ == 'valueRange':
            obj_ = Range.factory()
            obj_.build(child_)
            self.valueRange = obj_
            obj_.original_tagname_ = 'valueRange'
        elif nodeName_ == 'exclude':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.exclude = obj_
            obj_.original_tagname_ = 'exclude'
        super(Group_Characteristic, self).buildChildren(child_, node, nodeName_, True)
# end class Group_Characteristic


class GroupType(Element):
    """Types of resources that are part of groupIf the element is present,
    it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(GroupType, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if GroupType.subclass:
            return GroupType.subclass(*args_, **kwargs_)
        else:
            return GroupType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(GroupType, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='GroupType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='GroupType')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='GroupType', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='GroupType'):
        super(GroupType, self).exportAttributes(outfile, level, already_processed, namespace_, name_='GroupType')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='GroupType', fromsubclass_=False, pretty_print=True):
        super(GroupType, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='GroupType', mapping_=None):
        element = super(GroupType, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(GroupType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(GroupType, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class GroupType



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <falcon@gnu.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#    MODULE : Emergency Management
# 
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
#
#
# The documentation of the module goes in the "doc" directory.

from dateutil.relativedelta import relativedelta
from datetime import datetime, timedelta, date

from trytond.pyson import Eval, Not, Bool, PYSONEncoder, Equal
from trytond.model import ModelView, ModelSingleton, ModelSQL, fields, Unique
from trytond.pool import Pool


__all__ = [
    'GnuHealthSequences','Ambulance','SupportRequest', 'AmbulanceSupport',
    'AmbulanceHealthProfessional','SupportRequestLog']

class GnuHealthSequences(ModelSingleton, ModelSQL, ModelView):
    "Standard Sequences for GNU Health"
    __name__ = "gnuhealth.sequences"

    support_request_code_sequence = fields.Property(fields.Many2One('ir.sequence',
        'Support Request Sequence', 
        domain=[('code', '=', 'gnuhealth.support_request')],
        required=True))

class Ambulance (ModelSQL, ModelView):
    'Ambulance'
    __name__ = 'gnuhealth.ambulance'

    vehicle_identifier = fields.Char('ID', required=True,
        help="Ambulance license number or other type of ID")

    vehicle_brand = fields.Char('Brand', help="Ambulance maker")
    vehicle_model = fields.Char('Model', help="Ambulance model")

    vehicle_odometer = fields.Integer('Odometer',
        help="Current odometer reading")

    vehicle_type = fields.Selection([
        (None, ''),
        ('van', 'Van'),
        ('car', 'Car'),
        ('boat', 'Boat'),
        ('helicopter', 'Helicopter'),
        ('airplane', 'Airplane'),
        ('motorcycle', 'Motorcycle'),
        ('bicycle', 'Bicycle'),
        ('drone', 'Drone'),
        ], 'Type', required=True,
        help="Type of vehicle",sort=False)

    vehicle_function = fields.Selection([
        (None, ''),
        ('patient_transport', 'Type A - Patient Transport'),
        ('emergency', 'Type B - Emergency'),
        ('icu', 'Type C - Mobile ICU'),
        ], 'Function',sort=False, required=True, 
        help="Vehicle main functionality")
    
    vehicle_station = fields.Many2One(
        'gnuhealth.institution', 'Station',
        help="Station / Base of the vehicle")

    state = fields.Selection([
        (None, ''),
        ('available', 'Available / At Station'),
        ('dispatched', 'Dispatched'),
        ('en_route', 'En Route'),
        ('on_location', 'On Location'),
        ('to_hospital', 'To Hospital'),
        ('at_hospital', 'At Hospital'),
        ('returning', 'Returning'),
        ('out_of_service', 'Out of service'),
        ], 'Status',sort=False, readonly=True, help="Vehicle status")

    vehicle_remarks = fields.Text('Remarks')

    active = fields.Boolean('Active', select=True)

    @staticmethod
    def default_active():
        return True

    @staticmethod
    def default_state():
        return 'available'

    @classmethod
    def __setup__(cls):
        super(Ambulance, cls).__setup__()
        t = cls.__table__()
        cls._sql_constraints = [
            ('vehicle_uniq', Unique(t,t.vehicle_identifier), 
            'This vehicle ID already exists'),
        ]

    def get_rec_name(self, name):
        return (self.vehicle_identifier + ' : ' + self.vehicle_type)

class SupportRequest (ModelSQL, ModelView):
    'Support Request Registration'
    __name__ = 'gnuhealth.support_request'
    _rec_name = 'code'

    code = fields.Char('Code',help='Request Code', readonly=True)

    operator = fields.Many2One(
        'gnuhealth.healthprofessional', 'Operator',
        help="Operator who took the call / support request")

    requestor = fields.Many2One('party.party', 'Requestor',
    domain=[('is_person', '=', True)], help="Related party (person)")

    patient = fields.Many2One('gnuhealth.patient', 'Patient')

    evaluation = fields.Many2One('gnuhealth.patient.evaluation',
        'Evaluation', 
        domain=[('patient', '=', Eval('patient'))], depends=['patient'],
        help='Related Patient Evaluation')

    request_date = fields.DateTime('Date', required=True,
        help="Date and time of the call for help")
    
    operational_sector = fields.Many2One('gnuhealth.operational_sector',
        'O. Sector',help="Operational Sector")

    latitude = fields.Numeric('Latidude', digits=(3, 14))
    longitude = fields.Numeric('Longitude', digits=(4, 14))

    address = fields.Text("Address", help="Free text address / location")
    urladdr = fields.Char(
        'OSM Map',
        help="Maps the location on Open Street Map")

    healthcenter = fields.Many2One('gnuhealth.institution','Calling Institution')

    patient_sex = fields.Function(
        fields.Char('Sex'),
        'get_patient_sex')

    patient_age = fields.Function(
        fields.Char('Age'),
        'get_patient_age')

    complaint = fields.Function(
        fields.Char('Chief Complaint'),
        'get_patient_complaint')

    urgency = fields.Selection([
        (None, ''),
        ('low', 'Low'),
        ('urgent', 'Urgent'),
        ('emergency', 'Emergency'),
        ], 'Urgency', sort=False)
       
    place_occurrance = fields.Selection([
        (None, ''),
        ('home', 'Home'),
        ('street', 'Street'),
        ('institution', 'Institution'),
        ('school', 'School'),
        ('commerce', 'Commercial Area'),
        ('recreational', 'Recreational Area'),
        ('transportation', 'Public transportation'),
        ('sports', 'Sports event'),
        ('publicbuilding', 'Public Building'),
        ('unknown', 'Unknown'),
        ], 'Origin', help="Place of occurrance",sort=False)

    event_type = fields.Selection([
        (None, ''),
        ('event1', 'Acute Coronary Syndrome'),
        ('event2', 'Acute pain'),
        ('event3', 'Acute illness'),
        ('event4', 'Allergic reaction'),
        ('event5', 'Bullying, battering'),
        ('event6', 'Gastrointestinal event'),
        ('event7', 'Endocrine event (diabetes, adrenal crisis, ..)'),
        ('event8', 'Choke'),
        ('event9', 'Domestic violence'),
        ('event10', 'Environmental event (weather, animals, ...)'),
        ('event11', 'Sexual assault'),
        ('event12', 'Drug intoxication'),
        ('event13', 'Robbery, violent assault'),
        ('event14', 'Respiratory distress'),
        ('event15', 'Pregnancy related event'),
        ('event16', 'Gas intoxication'),
        ('event17', 'Food intoxication'),
        ('event18', 'Neurological event (stroke, TIA, seizure, ...)'),
        ('event19', 'Chronic illness'),
        ('event20', 'Near drowning'),
        ('event21', 'Eye, Ear and Nose event'),
        ('event22', 'Fall'),
        ('event23', 'Deceased person'),
        ('event24', 'Psychiatric event'),
        ('event25', 'Suicide attempt'),
        ('event26', 'Fire'),
        ('event27', 'Transportation accident'),
        ('event28', 'Traumatic Injuries'),
        ('event29', 'Explosion'),
        ('event30', 'Other specified'),
        ], 'Event type')

    event_specific = fields.Many2One ('gnuhealth.pathology','Incident')

    multiple_casualties = fields.Boolean('Multiple Casualties')

    request_actions = fields.One2Many(
        'gnuhealth.support_request.log', 'sr',
        'Activities', help='Support request activity log')

    ambulances = fields.One2Many(
        'gnuhealth.ambulance.support', 'sr',
        'Ambulances', help='Ambulances requested in this Support Request')

    request_extra_info = fields.Text('Details')

    state = fields.Selection([
        (None, ''),
        ('open', 'Open'),
        ('closed', 'Closed'),
        ], 'State', sort=False, readonly=True)
 
    @staticmethod
    def default_request_date():
        return datetime.now()

    
    def get_patient_sex(self, name):
        if self.patient:
            return self.patient.gender

    def get_patient_age(self, name):
        if self.patient:
            return self.patient.name.age

    def get_patient_complaint(self, name):
        if self.evaluation:
            if self.evaluation.chief_complaint:
                return self.evaluation.chief_complaint

    @staticmethod
    def default_operator():
        pool = Pool()
        HealthProf= pool.get('gnuhealth.healthprofessional')
        operator = HealthProf.get_health_professional()
        return operator

    @staticmethod
    def default_state():
        return 'open'


    @fields.depends('latitude', 'longitude')
    def on_change_with_urladdr(self):
        # Generates the URL to be used in OpenStreetMap
        # The address will be mapped to the URL in the following way
        # If the latitud and longitude of the Accident / Injury 
        # are given, then those parameters will be used.

        ret_url = ''
        if (self.latitude and self.longitude):
            ret_url = 'http://openstreetmap.org/?mlat=' + \
                str(self.latitude) + '&mlon=' + str(self.longitude)

        return ret_url

    @classmethod
    def create(cls, vlist):
        Sequence = Pool().get('ir.sequence')
        Config = Pool().get('gnuhealth.sequences')

        vlist = [x.copy() for x in vlist]
        for values in vlist:
            if not values.get('code'):
                config = Config(1)
                values['code'] = Sequence.get_id(
                    config.support_request_code_sequence.id)

        return super(SupportRequest, cls).create(vlist)


    @classmethod
    def __setup__(cls):
        super(SupportRequest, cls).__setup__()
        t = cls.__table__()
        cls._sql_constraints = [
            ('code_uniq', Unique(t,t.code), 
            'This Request Code already exists'),
        ]

        cls._buttons.update({
            'open_support': {'invisible': Equal(Eval('state'), 'open')},
            'close_support': {'invisible': Equal(Eval('state'), 'closed')},
            })


    @classmethod
    @ModelView.button
    def open_support(cls, srs):
        cls.write(srs, {
            'state': 'open'})

    @classmethod
    @ModelView.button
    def close_support(cls, srs):
        cls.write(srs, {
            'state': 'closed'})


class AmbulanceSupport (ModelSQL, ModelView):
    'Ambulance associated to a Support Request'
    __name__ = 'gnuhealth.ambulance.support'

    sr = fields.Many2One('gnuhealth.support_request',
        'SR', help="Support Request", required=True)

    ambulance = fields.Many2One('gnuhealth.ambulance','Ambulance',
        domain=[('state', '=', 'available')],)
    
    healthprofs = fields.One2Many('gnuhealth.ambulance_hp','name',
        'Health Professionals')

    state = fields.Selection([
        (None, ''),
        ('available', 'Available / At Station'),
        ('dispatched', 'Dispatched'),
        ('en_route', 'En Route'),
        ('on_location', 'On Location'),
        ('to_hospital', 'To Hospital'),
        ('at_hospital', 'At Hospital'),
        ('returning', 'Returning'),
        ('out_of_service', 'Out of service'),
        ], 'Status',sort=False, readonly=True, help="Vehicle status")

    @staticmethod
    def default_state():
        return 'available'


    @classmethod
    def __setup__(cls):
        super(AmbulanceSupport, cls).__setup__()
        cls._buttons.update({
            'available': {'invisible': Equal(Eval('state'), 'available')},
            'dispatched': {'invisible': Equal(Eval('state'), 'dispatched')},
            'en_route': {'invisible': Equal(Eval('state'), 'en_route')},
            'on_location': {'invisible': Equal(Eval('state'), 'on_location')},
            'to_hospital': {'invisible': Equal(Eval('state'), 'to_hospital')},
            'at_hospital': {'invisible': Equal(Eval('state'), 'at_hospital')},
            'returning': {'invisible': Equal(Eval('state'), 'returning')},
            'out_of_service': {'invisible': Equal(Eval('state'),
            'out_of_service')},
            })


    @classmethod
    @ModelView.button
    def available(cls, ambulances):
        cls.update_ambulance_status(ambulances, status='available')

    @classmethod
    @ModelView.button
    def dispatched(cls, ambulances):
        cls.update_ambulance_status(ambulances, status='dispatched')

    @classmethod
    @ModelView.button
    def en_route(cls, ambulances):
         cls.update_ambulance_status(ambulances, status='en_route')

    @classmethod
    @ModelView.button
    def on_location(cls, ambulances):
        cls.update_ambulance_status(ambulances, status='on_location')

    @classmethod
    @ModelView.button
    def to_hospital(cls, ambulances):
        cls.update_ambulance_status(ambulances, status='to_hospital')

    @classmethod
    @ModelView.button
    def at_hospital(cls, ambulances):
        cls.update_ambulance_status(ambulances, status='at_hospital')

    @classmethod
    @ModelView.button
    def returning(cls, ambulances):
        cls.update_ambulance_status(ambulances, status='returning')

    @classmethod
    @ModelView.button
    def out_of_service(cls, ambulances):
        cls.update_ambulance_status(ambulances, status='out_of_service')


    @classmethod
    def update_ambulance_status(cls, ambulances, status):
        # Update status on local support model for this ambulance
        cls.write(ambulances, {
            'state': status})
            
        # Write current status on ambulance model
        Ambulance = Pool().get('gnuhealth.ambulance')
        vehicle = []
        
        vehicle.append(ambulances[0].ambulance)
        
        Ambulance.write(vehicle, {
            'state': status })

        # Create a new current ambulance status on support request log
        Activity = Pool().get('gnuhealth.support_request.log')
        log = []
        timestamp = datetime.now()
        values = {
            'sr': 'ambulance',
            'action': 'ambulance',
            'remarks': status,
            'timestamp': timestamp,
            }
        values['sr'] = ambulances[0].sr
        
        log.append(values)
        Activity.create(log)


class AmbulanceHealthProfessional(ModelSQL, ModelView):
    'Ambulance Health Professionals'
    __name__ = 'gnuhealth.ambulance_hp'

    name = fields.Many2One('gnuhealth.ambulance.support', 'SR')

    healthprof = fields.Many2One(
        'gnuhealth.healthprofessional', 'Health Prof',
        help='Health Professional for this ambulance and support request')

class SupportRequestLog (ModelSQL, ModelView):
    'Ambulance associated to a Support Request'
    __name__ = 'gnuhealth.support_request.log'

    sr = fields.Many2One('gnuhealth.support_request',
        'SR', help="Support Request", required=True)

    timestamp = fields.DateTime('Time', required=True,
        help="Date and time of activity")

    action = fields.Selection([
        (None, ''),
        ('ambulance', 'Ambulance'),
        ('general', 'General'),
        ], 'Activity',sort=False, help="Activity log")

    remarks = fields.Char('Remarks', help="Remarks for this item")
    

    @staticmethod
    def default_timestamp():
        return datetime.now()

    @staticmethod
    def default_action():
        return 'general'



from .base_classes import *
from .support_functions import *


class FamilyHistory(Resource):
    """Significant health events and conditions for people related to the
    subject relevant in the context of care for the subject.If the
    element is present, it must have either a @value, an @id, or
    extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, subject=None, note=None, relation=None):
        self.original_tagname_ = None
        super(FamilyHistory, self).__init__(id, extension, modifierExtension, language, text, contained, )
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.subject = subject
        self.note = note
        if relation is None:
            self.relation = []
        else:
            self.relation = relation
    def factory(*args_, **kwargs_):
        if FamilyHistory.subclass:
            return FamilyHistory.subclass(*args_, **kwargs_)
        else:
            return FamilyHistory(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_note(self): return self.note
    def set_note(self, note): self.note = note
    def get_relation(self): return self.relation
    def set_relation(self, relation): self.relation = relation
    def add_relation(self, value): self.relation.append(value)
    def insert_relation(self, index, value): self.relation[index] = value
    def hasContent_(self):
        if (
            self.identifier or
            self.subject is not None or
            self.note is not None or
            self.relation or
            super(FamilyHistory, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='FamilyHistory', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='FamilyHistory')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='FamilyHistory', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='FamilyHistory'):
        super(FamilyHistory, self).exportAttributes(outfile, level, already_processed, namespace_, name_='FamilyHistory')
    def exportChildren(self, outfile, level, namespace_='', name_='FamilyHistory', fromsubclass_=False, pretty_print=True):
        super(FamilyHistory, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        if self.note is not None:
            self.note.export(outfile, level, namespace_, name_='note', pretty_print=pretty_print)
        for relation_ in self.relation:
            relation_.export(outfile, level, namespace_, name_='relation', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='FamilyHistory', mapping_=None):
        element = super(FamilyHistory, self).to_etree(parent_element, name_, mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        if self.note is not None:
            note_ = self.note
            note_.to_etree(element, name_='note', mapping_=mapping_)
        for relation_ in self.relation:
            relation_.to_etree(element, name_='relation', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(FamilyHistory, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'note':
            obj_ = string.factory()
            obj_.build(child_)
            self.note = obj_
            obj_.original_tagname_ = 'note'
        elif nodeName_ == 'relation':
            obj_ = FamilyHistory_Relation.factory()
            obj_.build(child_)
            self.relation.append(obj_)
            obj_.original_tagname_ = 'relation'
        super(FamilyHistory, self).buildChildren(child_, node, nodeName_, True)
# end class FamilyHistory


class FamilyHistory_Relation(BackboneElement):
    """Significant health events and conditions for people related to the
    subject relevant in the context of care for the subject.The
    actual or approximate date of birth of the relative.If this
    resource is indicating that the related person is deceased, then
    an indicator of whether the person is deceased (yes) or not (no)
    or the age or age range or description of age at death - can be
    indicated here. If the reason for death is known, then it can be
    indicated in the outcome code of the condition - in this case
    the deceased property should still be set."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, name=None, relationship=None, bornPeriod=None, bornDate=None, bornString=None, deceasedBoolean=None, deceasedAge=None, deceasedRange=None, deceasedDate=None, deceasedString=None, note=None, condition=None):
        self.original_tagname_ = None
        super(FamilyHistory_Relation, self).__init__(id, extension, modifierExtension, )
        self.name = name
        self.relationship = relationship
        self.bornPeriod = bornPeriod
        self.bornDate = bornDate
        self.bornString = bornString
        self.deceasedBoolean = deceasedBoolean
        self.deceasedAge = deceasedAge
        self.deceasedRange = deceasedRange
        self.deceasedDate = deceasedDate
        self.deceasedString = deceasedString
        self.note = note
        if condition is None:
            self.condition = []
        else:
            self.condition = condition
    def factory(*args_, **kwargs_):
        if FamilyHistory_Relation.subclass:
            return FamilyHistory_Relation.subclass(*args_, **kwargs_)
        else:
            return FamilyHistory_Relation(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_name(self): return self.name
    def set_name(self, name): self.name = name
    def get_relationship(self): return self.relationship
    def set_relationship(self, relationship): self.relationship = relationship
    def get_bornPeriod(self): return self.bornPeriod
    def set_bornPeriod(self, bornPeriod): self.bornPeriod = bornPeriod
    def get_bornDate(self): return self.bornDate
    def set_bornDate(self, bornDate): self.bornDate = bornDate
    def get_bornString(self): return self.bornString
    def set_bornString(self, bornString): self.bornString = bornString
    def get_deceasedBoolean(self): return self.deceasedBoolean
    def set_deceasedBoolean(self, deceasedBoolean): self.deceasedBoolean = deceasedBoolean
    def get_deceasedAge(self): return self.deceasedAge
    def set_deceasedAge(self, deceasedAge): self.deceasedAge = deceasedAge
    def get_deceasedRange(self): return self.deceasedRange
    def set_deceasedRange(self, deceasedRange): self.deceasedRange = deceasedRange
    def get_deceasedDate(self): return self.deceasedDate
    def set_deceasedDate(self, deceasedDate): self.deceasedDate = deceasedDate
    def get_deceasedString(self): return self.deceasedString
    def set_deceasedString(self, deceasedString): self.deceasedString = deceasedString
    def get_note(self): return self.note
    def set_note(self, note): self.note = note
    def get_condition(self): return self.condition
    def set_condition(self, condition): self.condition = condition
    def add_condition(self, value): self.condition.append(value)
    def insert_condition(self, index, value): self.condition[index] = value
    def hasContent_(self):
        if (
            self.name is not None or
            self.relationship is not None or
            self.bornPeriod is not None or
            self.bornDate is not None or
            self.bornString is not None or
            self.deceasedBoolean is not None or
            self.deceasedAge is not None or
            self.deceasedRange is not None or
            self.deceasedDate is not None or
            self.deceasedString is not None or
            self.note is not None or
            self.condition or
            super(FamilyHistory_Relation, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='FamilyHistory.Relation', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='FamilyHistory.Relation')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='FamilyHistory.Relation', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='FamilyHistory.Relation'):
        super(FamilyHistory_Relation, self).exportAttributes(outfile, level, already_processed, namespace_, name_='FamilyHistory.Relation')
    def exportChildren(self, outfile, level, namespace_='', name_='FamilyHistory.Relation', fromsubclass_=False, pretty_print=True):
        super(FamilyHistory_Relation, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.name is not None:
            self.name.export(outfile, level, namespace_, name_='name', pretty_print=pretty_print)
        if self.relationship is not None:
            self.relationship.export(outfile, level, namespace_, name_='relationship', pretty_print=pretty_print)
        if self.bornPeriod is not None:
            self.bornPeriod.export(outfile, level, namespace_, name_='bornPeriod', pretty_print=pretty_print)
        if self.bornDate is not None:
            self.bornDate.export(outfile, level, namespace_, name_='bornDate', pretty_print=pretty_print)
        if self.bornString is not None:
            self.bornString.export(outfile, level, namespace_, name_='bornString', pretty_print=pretty_print)
        if self.deceasedBoolean is not None:
            self.deceasedBoolean.export(outfile, level, namespace_, name_='deceasedBoolean', pretty_print=pretty_print)
        if self.deceasedAge is not None:
            self.deceasedAge.export(outfile, level, namespace_, name_='deceasedAge', pretty_print=pretty_print)
        if self.deceasedRange is not None:
            self.deceasedRange.export(outfile, level, namespace_, name_='deceasedRange', pretty_print=pretty_print)
        if self.deceasedDate is not None:
            self.deceasedDate.export(outfile, level, namespace_, name_='deceasedDate', pretty_print=pretty_print)
        if self.deceasedString is not None:
            self.deceasedString.export(outfile, level, namespace_, name_='deceasedString', pretty_print=pretty_print)
        if self.note is not None:
            self.note.export(outfile, level, namespace_, name_='note', pretty_print=pretty_print)
        for condition_ in self.condition:
            condition_.export(outfile, level, namespace_, name_='condition', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='FamilyHistory.Relation', mapping_=None):
        element = super(FamilyHistory_Relation, self).to_etree(parent_element, name_, mapping_)
        if self.name is not None:
            name_ = self.name
            name_.to_etree(element, name_='name', mapping_=mapping_)
        if self.relationship is not None:
            relationship_ = self.relationship
            relationship_.to_etree(element, name_='relationship', mapping_=mapping_)
        if self.bornPeriod is not None:
            bornPeriod_ = self.bornPeriod
            bornPeriod_.to_etree(element, name_='bornPeriod', mapping_=mapping_)
        if self.bornDate is not None:
            bornDate_ = self.bornDate
            bornDate_.to_etree(element, name_='bornDate', mapping_=mapping_)
        if self.bornString is not None:
            bornString_ = self.bornString
            bornString_.to_etree(element, name_='bornString', mapping_=mapping_)
        if self.deceasedBoolean is not None:
            deceasedBoolean_ = self.deceasedBoolean
            deceasedBoolean_.to_etree(element, name_='deceasedBoolean', mapping_=mapping_)
        if self.deceasedAge is not None:
            deceasedAge_ = self.deceasedAge
            deceasedAge_.to_etree(element, name_='deceasedAge', mapping_=mapping_)
        if self.deceasedRange is not None:
            deceasedRange_ = self.deceasedRange
            deceasedRange_.to_etree(element, name_='deceasedRange', mapping_=mapping_)
        if self.deceasedDate is not None:
            deceasedDate_ = self.deceasedDate
            deceasedDate_.to_etree(element, name_='deceasedDate', mapping_=mapping_)
        if self.deceasedString is not None:
            deceasedString_ = self.deceasedString
            deceasedString_.to_etree(element, name_='deceasedString', mapping_=mapping_)
        if self.note is not None:
            note_ = self.note
            note_.to_etree(element, name_='note', mapping_=mapping_)
        for condition_ in self.condition:
            condition_.to_etree(element, name_='condition', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(FamilyHistory_Relation, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'name':
            obj_ = string.factory()
            obj_.build(child_)
            self.name = obj_
            obj_.original_tagname_ = 'name'
        elif nodeName_ == 'relationship':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.relationship = obj_
            obj_.original_tagname_ = 'relationship'
        elif nodeName_ == 'bornPeriod':
            obj_ = Period.factory()
            obj_.build(child_)
            self.bornPeriod = obj_
            obj_.original_tagname_ = 'bornPeriod'
        elif nodeName_ == 'bornDate':
            obj_ = date.factory()
            obj_.build(child_)
            self.bornDate = obj_
            obj_.original_tagname_ = 'bornDate'
        elif nodeName_ == 'bornString':
            obj_ = string.factory()
            obj_.build(child_)
            self.bornString = obj_
            obj_.original_tagname_ = 'bornString'
        elif nodeName_ == 'deceasedBoolean':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.deceasedBoolean = obj_
            obj_.original_tagname_ = 'deceasedBoolean'
        elif nodeName_ == 'deceasedAge':
            obj_ = Age.factory()
            obj_.build(child_)
            self.deceasedAge = obj_
            obj_.original_tagname_ = 'deceasedAge'
        elif nodeName_ == 'deceasedRange':
            obj_ = Range.factory()
            obj_.build(child_)
            self.deceasedRange = obj_
            obj_.original_tagname_ = 'deceasedRange'
        elif nodeName_ == 'deceasedDate':
            obj_ = date.factory()
            obj_.build(child_)
            self.deceasedDate = obj_
            obj_.original_tagname_ = 'deceasedDate'
        elif nodeName_ == 'deceasedString':
            obj_ = string.factory()
            obj_.build(child_)
            self.deceasedString = obj_
            obj_.original_tagname_ = 'deceasedString'
        elif nodeName_ == 'note':
            obj_ = string.factory()
            obj_.build(child_)
            self.note = obj_
            obj_.original_tagname_ = 'note'
        elif nodeName_ == 'condition':
            obj_ = FamilyHistory_Condition.factory()
            obj_.build(child_)
            self.condition.append(obj_)
            obj_.original_tagname_ = 'condition'
        super(FamilyHistory_Relation, self).buildChildren(child_, node, nodeName_, True)
# end class FamilyHistory_Relation


class FamilyHistory_Condition(BackboneElement):
    """Significant health events and conditions for people related to the
    subject relevant in the context of care for the subject.Either
    the age of onset, range of approximate age or descriptive string
    can be recorded. For conditions with multiple occurrences, this
    describes the first known occurrence."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, type_=None, outcome=None, onsetAge=None, onsetRange=None, onsetString=None, note=None):
        self.original_tagname_ = None
        super(FamilyHistory_Condition, self).__init__(id, extension, modifierExtension, )
        self.type_ = type_
        self.outcome = outcome
        self.onsetAge = onsetAge
        self.onsetRange = onsetRange
        self.onsetString = onsetString
        self.note = note
    def factory(*args_, **kwargs_):
        if FamilyHistory_Condition.subclass:
            return FamilyHistory_Condition.subclass(*args_, **kwargs_)
        else:
            return FamilyHistory_Condition(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def get_outcome(self): return self.outcome
    def set_outcome(self, outcome): self.outcome = outcome
    def get_onsetAge(self): return self.onsetAge
    def set_onsetAge(self, onsetAge): self.onsetAge = onsetAge
    def get_onsetRange(self): return self.onsetRange
    def set_onsetRange(self, onsetRange): self.onsetRange = onsetRange
    def get_onsetString(self): return self.onsetString
    def set_onsetString(self, onsetString): self.onsetString = onsetString
    def get_note(self): return self.note
    def set_note(self, note): self.note = note
    def hasContent_(self):
        if (
            self.type_ is not None or
            self.outcome is not None or
            self.onsetAge is not None or
            self.onsetRange is not None or
            self.onsetString is not None or
            self.note is not None or
            super(FamilyHistory_Condition, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='FamilyHistory.Condition', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='FamilyHistory.Condition')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='FamilyHistory.Condition', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='FamilyHistory.Condition'):
        super(FamilyHistory_Condition, self).exportAttributes(outfile, level, already_processed, namespace_, name_='FamilyHistory.Condition')
    def exportChildren(self, outfile, level, namespace_='', name_='FamilyHistory.Condition', fromsubclass_=False, pretty_print=True):
        super(FamilyHistory_Condition, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        if self.outcome is not None:
            self.outcome.export(outfile, level, namespace_, name_='outcome', pretty_print=pretty_print)
        if self.onsetAge is not None:
            self.onsetAge.export(outfile, level, namespace_, name_='onsetAge', pretty_print=pretty_print)
        if self.onsetRange is not None:
            self.onsetRange.export(outfile, level, namespace_, name_='onsetRange', pretty_print=pretty_print)
        if self.onsetString is not None:
            self.onsetString.export(outfile, level, namespace_, name_='onsetString', pretty_print=pretty_print)
        if self.note is not None:
            self.note.export(outfile, level, namespace_, name_='note', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='FamilyHistory.Condition', mapping_=None):
        element = super(FamilyHistory_Condition, self).to_etree(parent_element, name_, mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        if self.outcome is not None:
            outcome_ = self.outcome
            outcome_.to_etree(element, name_='outcome', mapping_=mapping_)
        if self.onsetAge is not None:
            onsetAge_ = self.onsetAge
            onsetAge_.to_etree(element, name_='onsetAge', mapping_=mapping_)
        if self.onsetRange is not None:
            onsetRange_ = self.onsetRange
            onsetRange_.to_etree(element, name_='onsetRange', mapping_=mapping_)
        if self.onsetString is not None:
            onsetString_ = self.onsetString
            onsetString_.to_etree(element, name_='onsetString', mapping_=mapping_)
        if self.note is not None:
            note_ = self.note
            note_.to_etree(element, name_='note', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(FamilyHistory_Condition, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'type':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'outcome':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.outcome = obj_
            obj_.original_tagname_ = 'outcome'
        elif nodeName_ == 'onsetAge':
            obj_ = Age.factory()
            obj_.build(child_)
            self.onsetAge = obj_
            obj_.original_tagname_ = 'onsetAge'
        elif nodeName_ == 'onsetRange':
            obj_ = Range.factory()
            obj_.build(child_)
            self.onsetRange = obj_
            obj_.original_tagname_ = 'onsetRange'
        elif nodeName_ == 'onsetString':
            obj_ = string.factory()
            obj_.build(child_)
            self.onsetString = obj_
            obj_.original_tagname_ = 'onsetString'
        elif nodeName_ == 'note':
            obj_ = string.factory()
            obj_.build(child_)
            self.note = obj_
            obj_.original_tagname_ = 'note'
        super(FamilyHistory_Condition, self).buildChildren(child_, node, nodeName_, True)
# end class FamilyHistory_Condition
